@@ -19,6 +19,7 @@
  *
  */
 
+#include <errno.h>
 #include <X11/keysym.h>
 
 #include "config.h"
@@ -27,12 +28,10 @@
 #include "rules.h"
 #include "screen.h"
 #include "widget.h"
-#include "xutil.h"
-#include "ewmh.h"
 #include "defconfig.h"
 #include "layouts/tile.h"
 #include "common/configopts.h"
-#include "common/configopts.h"
+#include "common/xutil.h"
 
 /* Permit to use mouse with many more buttons */
 #ifndef Button6
@@ -49,7 +48,7 @@ extern cfg_opt_t awesome_opts[];
 typedef struct
 {
     const char *name;
-    KeyCode keycode;
+    KeySym keysym;
 } KeyMod;
 
 /** Link a name to a mouse button symbol */
@@ -68,7 +67,7 @@ extern const name_func_link_t FloatingPlacementList[];
  * \param keyname Key name
  * \return Key mask or 0 if not found
  */
-static KeyCode
+static KeySym
 key_mask_lookup(const char *keyname)
 {
     /** List of keyname and corresponding X11 mask codes */
@@ -89,7 +88,7 @@ key_mask_lookup(const char *keyname)
     if(keyname)
         for(i = 0; KeyModList[i].name; i++)
             if(!a_strcmp(keyname, KeyModList[i].name))
-                return KeyModList[i].keycode;
+                return KeyModList[i].keysym;
 
     return 0;
 }
@@ -166,18 +165,22 @@ set_key_info(Key *key, cfg_t *cfg)
         warn("unknown command %s\n", cfg_getstr(cfg, "command"));
 }
 
-static KeyCode
-key_to_keycode(char *str)
+static void
+config_key_store(Key *key, char *str)
 {
     KeyCode kc;
     int ikc;
 
-    if(a_strncmp(str, "#", 1))
-        return XKeysymToKeycode(globalconf.display, XStringToKeysym(str));
-
-    ikc = atoi(str + 1);
-    memcpy(&kc, &ikc, sizeof(KeyCode));
-    return kc;
+    if(!a_strlen(str))
+        return;
+    else if(a_strncmp(str, "#", 1))
+        key->keysym = XStringToKeysym(str);
+    else
+    {
+        ikc = atoi(str + 1);
+        memcpy(&kc, &ikc, sizeof(KeyCode));
+        key->keycode = kc;
+    }
 }
 
 static Key *
@@ -192,7 +195,7 @@ section_keys(cfg_t *cfg_keys)
         key = p_new(Key, 1);
         cfgkeytmp = cfg_getnsec(cfg_keys, "key", i);
         set_key_info(key, cfgkeytmp);
-        key->keycode = key_to_keycode(cfg_getstr(cfgkeytmp, "key"));
+        config_key_store(key, cfg_getstr(cfgkeytmp, "key"));
         key->arg = a_strdup(cfg_getstr(cfgkeytmp, "arg"));
         key_list_push(&head, key);
     }
@@ -211,7 +214,7 @@ section_keys(cfg_t *cfg_keys)
         {
             key = p_new(Key, 1);
             set_key_info(key, cfgkeytmp);
-            key->keycode = key_to_keycode(cfg_getnstr(cfgkeytmp, "keylist", j));
+            config_key_store(key, cfg_getnstr(cfgkeytmp, "keylist", j));
             key->arg = a_strdup(cfg_getnstr(cfgkeytmp, "arglist", j));
             key_list_push(&head, key);
         }
@@ -234,7 +237,7 @@ cmp_widget_cfg(const void *a, const void *b)
 }
 
 static void
-create_widgets(cfg_t* cfg_statusbar, Statusbar *statusbar)
+statusbar_widgets_create(cfg_t *cfg_statusbar, Statusbar *statusbar)
 {
     cfg_t* widgets, *wptr;
     Widget *widget = NULL;
@@ -244,8 +247,7 @@ create_widgets(cfg_t* cfg_statusbar, Statusbar *statusbar)
     for(i = 0; WidgetList[i].name; i++)
         numwidgets += cfg_size(cfg_statusbar, WidgetList[i].name);
 
-    widgets = p_new(cfg_t, numwidgets);
-    wptr = widgets;
+    wptr = widgets = p_new(cfg_t, numwidgets);
 
     for(i = 0; WidgetList[i].name; i++)
         for (j = 0; j < cfg_size(cfg_statusbar, WidgetList[i].name); j++)
@@ -260,17 +262,44 @@ create_widgets(cfg_t* cfg_statusbar, Statusbar *statusbar)
 
     for(i = 0; i < numwidgets; i++)
     {
-        wptr = widgets + i;
+        wptr = &widgets[i];
         widget_new = name_func_lookup(cfg_name(wptr), WidgetList);
         if(widget_new)
         {
             widget = widget_new(statusbar, wptr);
             widget_list_append(&statusbar->widgets, widget);
-            widget->buttons = parse_mouse_bindings(wptr, "mouse", a_strcmp(cfg_name(wptr), "taglist"));
+            widget->buttons = parse_mouse_bindings(wptr, "mouse",
+                                                   a_strcmp(cfg_name(wptr), "taglist") ? True : False);
         }
         else
-            warn("Ignoring unknown widget: %s.\n", cfg_name(widgets + i));
+            warn("ignoring unknown widget: %s.\n", cfg_name(widgets + i));
     }
+    p_delete(&widgets);
+}
+
+static void
+config_section_titlebar_init(cfg_t *cfg_titlebar, Titlebar *tb, int screen)
+{
+    int phys_screen = screen_virttophys(screen);
+    cfg_t *cfg_styles = cfg_getsec(cfg_titlebar, "styles");
+
+    tb->position = tb->dposition = cfg_getposition(cfg_titlebar, "position");
+    tb->align = cfg_getalignment(cfg_titlebar, "align");
+    tb->text_align = cfg_getalignment(cfg_titlebar, "text_align");
+    tb->width = cfg_getint(cfg_titlebar, "width");
+    tb->height = cfg_getint(cfg_titlebar, "height");
+    draw_style_init(globalconf.display, phys_screen,
+                    cfg_getsec(cfg_styles, "normal"),
+                    &tb->styles.normal,
+                    &globalconf.screens[screen].styles.normal);
+    draw_style_init(globalconf.display, phys_screen,
+                    cfg_getsec(cfg_styles, "focus"),
+                    &tb->styles.focus,
+                    &globalconf.screens[screen].styles.focus);
+    draw_style_init(globalconf.display, phys_screen,
+                    cfg_getsec(cfg_styles, "urgent"),
+                    &tb->styles.urgent,
+                    &globalconf.screens[screen].styles.urgent);
 }
 
 static void
@@ -282,10 +311,11 @@ config_parse_screen(cfg_t *cfg, int screen)
     Layout *layout = NULL;
     Tag *tag = NULL;
     Statusbar *statusbar = NULL;
-    cfg_t *cfg_general, *cfg_colors, *cfg_screen, *cfg_tags,
-          *cfg_layouts, *cfg_padding, *cfgsectmp;
+    cfg_t *cfg_general, *cfg_styles, *cfg_screen, *cfg_tags,
+          *cfg_layouts, *cfg_padding, *cfgsectmp, *cfg_titlebar,
+          *cfg_styles_normal, *cfg_styles_focus, *cfg_styles_urgent;
     VirtScreen *virtscreen = &globalconf.screens[screen];
-    int i, phys_screen = get_phys_screen(screen);
+    int i, phys_screen = screen_virttophys(screen);
 
     snprintf(buf, sizeof(buf), "%d", screen);
     cfg_screen = cfg_gettsec(cfg, "screen", buf);
@@ -301,8 +331,9 @@ config_parse_screen(cfg_t *cfg, int screen)
 
     /* get screen specific sections */
     cfg_tags = cfg_getsec(cfg_screen, "tags");
-    cfg_colors = cfg_getsec(cfg_screen, "colors");
+    cfg_styles = cfg_getsec(cfg_screen, "styles");
     cfg_general = cfg_getsec(cfg_screen, "general");
+    cfg_titlebar = cfg_getsec(cfg_screen, "titlebar");
     cfg_layouts = cfg_getsec(cfg_screen, "layouts");
     cfg_padding = cfg_getsec(cfg_screen, "padding");
 
@@ -315,13 +346,20 @@ config_parse_screen(cfg_t *cfg, int screen)
     virtscreen->sloppy_focus_raise = cfg_getbool(cfg_general, "sloppy_focus_raise");
     virtscreen->new_become_master = cfg_getbool(cfg_general, "new_become_master");
     virtscreen->new_get_focus = cfg_getbool(cfg_general, "new_get_focus");
-    virtscreen->opacity_unfocused = cfg_getint(cfg_general, "opacity_unfocused");
-    virtscreen->font = XftFontOpenName(globalconf.display,
-                                       phys_screen,
-                                       cfg_getstr(cfg_general, "font"));
+    virtscreen->opacity_unfocused = cfg_getfloat(cfg_general, "opacity_unfocused");
+    virtscreen->opacity_focused = cfg_getfloat(cfg_general, "opacity_focused");
     virtscreen->floating_placement =
         name_func_lookup(cfg_getstr(cfg_general, "floating_placement"),
                                     FloatingPlacementList);
+    virtscreen->mwfact_lower_limit = cfg_getfloat(cfg_general, "mwfact_lower_limit");
+    virtscreen->mwfact_upper_limit = cfg_getfloat(cfg_general, "mwfact_upper_limit");
+
+    if(virtscreen->mwfact_upper_limit < virtscreen->mwfact_lower_limit)
+    {
+        warn("mwfact_upper_limit must be greater than mwfact_lower_limit\n");
+        virtscreen->mwfact_upper_limit = 0.9;
+        virtscreen->mwfact_lower_limit = 0.1;
+    }
 
     if(!virtscreen->floating_placement)
     {
@@ -329,50 +367,44 @@ config_parse_screen(cfg_t *cfg, int screen)
         virtscreen->floating_placement = FloatingPlacementList[0].func;
     }
 
-    if(!virtscreen->font)
-        eprint("awesome: cannot init font\n");
-
     /* Colors */
-    draw_color_new(globalconf.display, phys_screen,
-                   cfg_getstr(cfg_colors, "normal_border"),
-                   &virtscreen->colors_normal[ColBorder]);
-    draw_color_new(globalconf.display, phys_screen,
-                   cfg_getstr(cfg_colors, "normal_bg"),
-                   &virtscreen->colors_normal[ColBG]);
-    draw_color_new(globalconf.display, phys_screen,
-                   cfg_getstr(cfg_colors, "normal_fg"),
-                   &virtscreen->colors_normal[ColFG]);
-    draw_color_new(globalconf.display, phys_screen,
-                   cfg_getstr(cfg_colors, "focus_border"),
-                   &virtscreen->colors_selected[ColBorder]);
-    draw_color_new(globalconf.display, phys_screen,
-                   cfg_getstr(cfg_colors, "focus_bg"),
-                   &virtscreen->colors_selected[ColBG]);
-    draw_color_new(globalconf.display, phys_screen,
-                   cfg_getstr(cfg_colors, "focus_fg"),
-                   &virtscreen->colors_selected[ColFG]);
-    draw_color_new(globalconf.display, phys_screen,
-                   cfg_getstr(cfg_colors, "urgent_bg"),
-                   &virtscreen->colors_urgent[ColBG]);
-    draw_color_new(globalconf.display, phys_screen,
-                   cfg_getstr(cfg_colors, "urgent_fg"),
-                   &virtscreen->colors_urgent[ColFG]);
+    if(!cfg_styles)
+        eprint("no colors section found");
+
+    if(!(cfg_styles_normal = cfg_getsec(cfg_styles, "normal")))
+       eprint("no normal colors section found\n");
+    if(!(cfg_styles_focus = cfg_getsec(cfg_styles, "focus")))
+       eprint("no focus colors section found\n");
+    if(!(cfg_styles_urgent = cfg_getsec(cfg_styles, "urgent")))
+       eprint("no urgent colors section found\n");
+
+    draw_style_init(globalconf.display, phys_screen,
+                    cfg_styles_normal, &virtscreen->styles.normal, NULL);
+    draw_style_init(globalconf.display, phys_screen,
+                    cfg_styles_focus, &virtscreen->styles.focus, &virtscreen->styles.normal);
+    draw_style_init(globalconf.display, phys_screen,
+                    cfg_styles_urgent, &virtscreen->styles.urgent, &virtscreen->styles.normal);
+
+    if(!virtscreen->styles.normal.font)
+        eprint("no font available\n");
+
+    /* Titlebar */
+    config_section_titlebar_init(cfg_titlebar, &virtscreen->titlebar_default, screen);
 
     /* Statusbar */
-
     statusbar_list_init(&virtscreen->statusbar);
     for(i = cfg_size(cfg_screen, "statusbar") - 1; i >= 0; i--)
     {
         statusbar = p_new(Statusbar, 1);
         cfgsectmp = cfg_getnsec(cfg_screen, "statusbar", i);
         statusbar->position = statusbar->dposition =
-            statusbar_get_position_from_str(cfg_getstr(cfgsectmp, "position"));
+            cfg_getposition(cfgsectmp, "position");
         statusbar->height = cfg_getint(cfgsectmp, "height");
         statusbar->width = cfg_getint(cfgsectmp, "width");
         statusbar->name = a_strdup(cfg_title(cfgsectmp));
         statusbar->screen = screen;
         statusbar_preinit(statusbar);
-        create_widgets(cfgsectmp, statusbar);
+        statusbar_widgets_create(cfgsectmp, statusbar);
         statusbar_list_push(&virtscreen->statusbar, statusbar);
     }
 
@@ -431,10 +463,6 @@ config_parse_screen(cfg_t *cfg, int screen)
         tag_push_to_screen(tag, screen);
     }
 
-    ewmh_update_net_numbers_of_desktop(phys_screen);
-    ewmh_update_net_current_desktop(phys_screen);
-    ewmh_update_net_desktop_names(phys_screen);
-
     /* select first tag by default */
     virtscreen->tags[0].selected = True;
     virtscreen->tags[0].was_selected = True;
@@ -465,19 +493,19 @@ config_parse(const char *confpatharg)
 
     globalconf.configpath = a_strdup(confpath);
 
-    cfg = cfg_init(awesome_opts, CFGF_NONE);
+    cfg = cfg_new();
 
     ret = cfg_parse(cfg, confpath);
 
     switch(ret)
     {
       case CFG_FILE_ERROR:
-        perror("awesome: parsing configuration file failed");
+        warn("parsing configuration file failed: %s\n", strerror(errno));
         if(!(defconfig = fopen(confpath, "w")))
-           perror("awesome: unable to create default configuration file");
+           warn("unable to create default configuration file: %s\n", strerror(errno));
         break;
       case CFG_PARSE_ERROR:
-        cfg_error(cfg, "awesome: parsing configuration file %s failed.\n", confpath);
+        cfg_error(cfg, "W: awesome: parsing configuration file %s failed.\n", confpath);
         break;
     }
 
@@ -490,7 +518,7 @@ config_parse(const char *confpatharg)
     }
 
     /* get the right screen section */
-    for(screen = 0; screen < globalconf.nscreen; screen++)
+    for(screen = 0; screen < globalconf.screens_info->nscreen; screen++)
         config_parse_screen(cfg, screen);
 
     /* get general sections */
@@ -513,7 +541,8 @@ config_parse(const char *confpatharg)
         rule->screen = cfg_getint(cfgsectmp, "screen");
         rule->ismaster = rules_get_fuzzy_from_str(cfg_getstr(cfgsectmp, "master"));
         rule->opacity = cfg_getfloat(cfgsectmp, "opacity");
-        if(rule->screen >= globalconf.nscreen)
+        config_section_titlebar_init(cfg_getsec(cfgsectmp, "titlebar"), &rule->titlebar, 0);
+        if(rule->screen >= globalconf.screens_info->nscreen)
             rule->screen = 0;
 
         rule_list_push(&globalconf.rules, rule);
@@ -525,8 +554,11 @@ config_parse(const char *confpatharg)
     /* Mouse: client windows click bindings */
     globalconf.buttons.client = parse_mouse_bindings(cfg_mouse, "client", True);
 
+    /* Mouse: titlebar windows click bindings */
+    globalconf.buttons.titlebar = parse_mouse_bindings(cfg_mouse, "titlebar", True);
+
     /* Keys */
-    globalconf.numlockmask = get_numlockmask(globalconf.display);
+    globalconf.numlockmask = xgetnumlockmask(globalconf.display);
 
     globalconf.keys = section_keys(cfg_keys);
 
