@@ -1,7 +1,7 @@
 /*
  * layout.c - layout management
  *
- * Copyright © 2007 Julien Danjou <julien@danjou.info>
+ * Copyright © 2007-2008 Julien Danjou <julien@danjou.info>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,7 +23,6 @@
 #include <X11/Xutil.h>
 
 #include "tag.h"
-#include "xutil.h"
 #include "focus.h"
 #include "widget.h"
 #include "window.h"
@@ -47,7 +46,7 @@ arrange(int screen)
     Client *c;
     Layout *curlay = layout_get_current(screen);
     unsigned int dui;
-    int di, x, y;
+    int di, x, y, phys_screen = screen_virttophys(screen);
     Window rootwin, childwin;
 
     for(c = globalconf.clients; c; c = c->next)
@@ -66,21 +65,30 @@ arrange(int screen)
         {
             c->newcomer = False;
             client_unban(c);
-            if(globalconf.screens[screen].new_get_focus)
+            if(globalconf.screens[screen].new_get_focus
+               && !c->skip
+               && (!globalconf.focus->client || !globalconf.focus->client->ismax))
                 client_focus(c, screen, True);
+            else if(globalconf.focus->client && globalconf.focus->client->ismax)
+                client_stack(globalconf.focus->client);
         }
 
     /* if we have a valid client that could be focused but currently no window
      * are focused, then set the focus on this window */
-    if((c = focus_get_current_client(screen)) && !globalconf.focus->client)
+    if((c = focus_get_current_client(screen)) && globalconf.focus->client != c)
         client_focus(c, screen, True);
 
     /* check that the mouse is on a window or not */
-    if(XQueryPointer(globalconf.display, RootWindow(globalconf.display,
-                                                    get_phys_screen(screen)),
-                     &rootwin, &childwin, &x, &y, &di, &di, &dui)
-       && (rootwin == None || childwin == None || childwin == rootwin))
-        window_root_grabbuttons(get_phys_screen(screen));
+    if(XQueryPointer(globalconf.display,
+                     RootWindow(globalconf.display, phys_screen),
+                     &rootwin, &childwin, &x, &y, &di, &di, &dui))
+    {
+       if(rootwin == None || childwin == None || childwin == rootwin)
+           window_root_grabbuttons(phys_screen);
+
+       globalconf.pointer_x = x;
+       globalconf.pointer_y = y;
+    }
 
     /* reset status */
     globalconf.screens[screen].need_arrange = False;
@@ -95,7 +103,7 @@ layout_refresh(void)
     int screen;
     int arranged = 0;
 
-    for(screen = 0; screen < globalconf.nscreen; screen++)
+    for(screen = 0; screen < globalconf.screens_info->nscreen; screen++)
         if(globalconf.screens[screen].need_arrange)
         {
             arrange(screen);
@@ -118,7 +126,8 @@ layout_get_current(int screen)
     return l;
 }
 
-/** Set layout for tag
+/** Set the layout of the current tag.
+ * Argument must be a relative or absolute integer of available layouts.
  * \param screen Screen ID
  * \param arg Layout specifier
  * \ingroup ui_callback
